@@ -1,4 +1,4 @@
-/* $Id: statusbar_gui.cpp 25290 2013-05-26 19:25:01Z frosch $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -24,13 +24,17 @@
 #include "saveload/saveload.h"
 #include "window_func.h"
 #include "statusbar_gui.h"
+#include "toolbar_gui.h"
 #include "core/geometry_func.hpp"
+#include "guitimer_func.h"
 
 #include "widgets/statusbar_widget.h"
 
 #include "table/strings.h"
 #include "table/sprites.h"
 
+#include "safeguards.h"
+
 static bool DrawScrollingStatusText(const NewsItem *ni, int scroll_pos, int left, int right, int top, int bottom)
 {
 	CopyInDParam(0, ni->params, lengthof(ni->params));
@@ -76,17 +80,19 @@ static bool DrawScrollingStatusText(const NewsItem *ni, int scroll_pos, int left
 struct StatusBarWindow : Window {
 	bool saving;
 	int ticker_scroll;
-	int reminder_timeout;
+	GUITimer ticker_timer;
+	GUITimer reminder_timeout;
 
 	static const int TICKER_STOP    = 1640; ///< scrolling is finished when counter reaches this value
-	static const int REMINDER_START =   91; ///< initial value of the reminder counter (right dot on the right)
+	static const int REMINDER_START = 1350; ///< time in ms for reminder notification (red dot on the right) to stay
 	static const int REMINDER_STOP  =    0; ///< reminder disappears when counter reaches this value
 	static const int COUNTER_STEP   =    2; ///< this is subtracted from active counters every tick
 
 	StatusBarWindow(WindowDesc *desc) : Window(desc)
 	{
-		this->ticker_scroll    =   TICKER_STOP;
-		this->reminder_timeout = REMINDER_STOP;
+		this->ticker_scroll = TICKER_STOP;
+		this->ticker_timer.SetInterval(15);
+		this->reminder_timeout.SetInterval(REMINDER_STOP);
 
 		this->InitNested();
 		CLRBITS(this->flags, WF_WHITE_BORDER);
@@ -99,6 +105,11 @@ struct StatusBarWindow : Window {
 		return pt;
 	}
 
+	virtual void FindWindowPlacementAndResize(int def_width, int def_height)
+	{
+		Window::FindWindowPlacementAndResize(_toolbar_width, def_height);
+	}
+
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
 		Dimension d;
@@ -171,7 +182,7 @@ struct StatusBarWindow : Window {
 					}
 				}
 
-				if (this->reminder_timeout > 0) {
+				if (!this->reminder_timeout.HasElapsed()) {
 					Dimension icon_size = GetSpriteSize(SPR_UNREAD_NEWS);
 					DrawSprite(SPR_UNREAD_NEWS, PAL_NONE, r.right - WD_FRAMERECT_RIGHT - icon_size.width, r.top + WD_FRAMERECT_TOP + (int)(FONT_HEIGHT_NORMAL - icon_size.height) / 2);
 				}
@@ -192,10 +203,10 @@ struct StatusBarWindow : Window {
 			case SBI_SAVELOAD_START:  this->saving = true;  break;
 			case SBI_SAVELOAD_FINISH: this->saving = false; break;
 			case SBI_SHOW_TICKER:     this->ticker_scroll = 0; break;
-			case SBI_SHOW_REMINDER:   this->reminder_timeout = REMINDER_START; break;
+			case SBI_SHOW_REMINDER:   this->reminder_timeout.SetInterval(REMINDER_START); break;
 			case SBI_NEWS_DELETED:
 				this->ticker_scroll    =   TICKER_STOP; // reset ticker ...
-				this->reminder_timeout = REMINDER_STOP; // ... and reminder
+				this->reminder_timeout.SetInterval(REMINDER_STOP); // ... and reminder
 				break;
 		}
 	}
@@ -209,19 +220,20 @@ struct StatusBarWindow : Window {
 		}
 	}
 
-	virtual void OnTick()
+	virtual void OnRealtimeTick(uint delta_ms)
 	{
 		if (_pause_mode != PM_UNPAUSED) return;
 
 		if (this->ticker_scroll < TICKER_STOP) { // Scrolling text
-			this->ticker_scroll += COUNTER_STEP;
-			this->SetWidgetDirty(WID_S_MIDDLE);
+			uint count = this->ticker_timer.CountElapsed(delta_ms);
+			if (count > 0) {
+				this->ticker_scroll += count;
+				this->SetWidgetDirty(WID_S_MIDDLE);
+			}
 		}
 
-		if (this->reminder_timeout > REMINDER_STOP) { // Red blot to show there are new unread newsmessages
-			this->reminder_timeout -= COUNTER_STEP;
-		} else if (this->reminder_timeout < REMINDER_STOP) {
-			this->reminder_timeout = REMINDER_STOP;
+		// Red blot to show there are new unread newsmessages
+		if (this->reminder_timeout.Elapsed(delta_ms)) {
 			this->SetWidgetDirty(WID_S_MIDDLE);
 		}
 	}
@@ -236,7 +248,7 @@ static const NWidgetPart _nested_main_status_widgets[] = {
 };
 
 static WindowDesc _main_status_desc(
-	WDP_MANUAL, NULL, 640, 12,
+	WDP_MANUAL, NULL, 0, 0,
 	WC_STATUS_BAR, WC_NONE,
 	WDF_NO_FOCUS,
 	_nested_main_status_widgets, lengthof(_nested_main_status_widgets)
