@@ -1,4 +1,4 @@
-/* $Id: stdafx.h 27380 2015-08-10 20:21:29Z michi_cc $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -100,12 +100,6 @@
 	#define strcasecmp stricmp
 #endif
 
-#if defined(PSP)
-	#include <psptypes.h>
-	#include <pspdebug.h>
-	#include <pspthreadman.h>
-#endif
-
 #if defined(SUNOS) || defined(HPUX)
 	#include <alloca.h>
 #endif
@@ -134,20 +128,11 @@
 	#define CLIB_USERGROUP_PROTOS_H
 #endif /* __MORPHOS__ */
 
-#if defined(PSP)
-	/* PSP can only have 10 file-descriptors open at any given time, but this
-	 *  switch only limits reads via the Fio system. So keep 2 fds free for things
-	 *  like saving a game. */
-	#define LIMITED_FDS 8
-	#define printf pspDebugScreenPrintf
-#endif /* PSP */
-
 /* Stuff for GCC */
 #if defined(__GNUC__)
 	#define NORETURN __attribute__ ((noreturn))
 	#define CDECL
 	#define __int64 long long
-	#define GCC_PACK __attribute__((packed))
 	/* Warn about functions using 'printf' format syntax. First argument determines which parameter
 	 * is the format string, second argument is start of values passed to printf. */
 	#define WARN_FORMAT(string, args) __attribute__ ((format (printf, string, args)))
@@ -156,14 +141,25 @@
 	#else
 		#define FINAL
 	#endif
+
+	/* Use fallthrough attribute where supported */
+	#if __GNUC__ >= 7
+		#if __cplusplus > 201402L // C++17
+			#define FALLTHROUGH [[fallthrough]]
+		#else
+			#define FALLTHROUGH __attribute__((fallthrough))
+		#endif
+	#else
+		#define FALLTHROUGH
+	#endif
 #endif /* __GNUC__ */
 
 #if defined(__WATCOMC__)
 	#define NORETURN
 	#define CDECL
-	#define GCC_PACK
 	#define WARN_FORMAT(string, args)
 	#define FINAL
+	#define FALLTHROUGH
 	#include <malloc.h>
 #endif /* __WATCOMC__ */
 
@@ -171,7 +167,7 @@
 	#include <malloc.h> // alloca()
 #endif
 
-#if defined(WIN32)
+#if defined(_WIN32)
 	#define WIN32_LEAN_AND_MEAN     // Exclude rarely-used stuff from Windows headers
 #endif
 
@@ -191,9 +187,7 @@
 		#define NTDDI_VERSION NTDDI_WIN2K // Windows 2000
 		#define _WIN32_WINNT 0x0500       // Windows 2000
 		#define _WIN32_WINDOWS 0x400      // Windows 95
-		#if !defined(WINCE)
-			#define WINVER 0x0400     // Windows NT 4.0 / Windows 95
-		#endif
+		#define WINVER 0x0400             // Windows NT 4.0 / Windows 95
 		#define _WIN32_IE_ 0x0401         // 4.01 (win98 and NT4SP5+)
 	#endif
 	#define NOMINMAX                // Disable min/max macros in windows.h.
@@ -227,47 +221,46 @@
 		#define inline __forceinline
 	#endif
 
-	#if !defined(WINCE)
-		#define CDECL _cdecl
-	#endif
-
-	#define GCC_PACK
+	#define CDECL _cdecl
 	#define WARN_FORMAT(string, args)
 	#define FINAL sealed
 
-	#if defined(WINCE)
-		int CDECL vsnprintf(char *str, size_t size, const char *format, va_list ap);
+	/* fallthrough attribute, VS 2017 */
+	#if (_MSC_VER >= 1910)
+		#define FALLTHROUGH [[fallthrough]]
+	#else
+		#define FALLTHROUGH
 	#endif
 
-	#if defined(WIN32) && !defined(_WIN64) && !defined(WIN64)
+#	if defined(_WIN32) && !defined(_WIN64)
 		#if !defined(_W64)
 			#define _W64
 		#endif
 
 		typedef _W64 int INT_PTR, *PINT_PTR;
 		typedef _W64 unsigned int UINT_PTR, *PUINT_PTR;
-	#endif /* WIN32 && !_WIN64 && !WIN64 */
+#	endif /* _WIN32 && !_WIN64 */
 
-	#if defined(_WIN64) || defined(WIN64)
+#	if defined(_WIN64)
 		#define fseek _fseeki64
-	#endif /* _WIN64 || WIN64 */
+#	endif /* _WIN64 */
 
-	/* This is needed to zlib uses the stdcall calling convention on visual studio */
-	#if defined(WITH_ZLIB) || defined(WITH_PNG)
-		#if !defined(ZLIB_WINAPI)
-			#define ZLIB_WINAPI
-		#endif
-	#endif
+	/* zlib from vcpkg use cdecl calling convention without enforcing it in the headers */
+#	if defined(WITH_ZLIB)
+#		if !defined(ZEXPORT)
+#			define ZEXPORT CDECL
+#		endif
+#	endif
 
-	#if defined(WINCE)
-		#define strcasecmp _stricmp
-		#define strncasecmp _strnicmp
-		#undef DEBUG
-	#else
-		#define strcasecmp stricmp
-		#define strncasecmp strnicmp
-	#endif
+	/* freetype from vcpkg use cdecl calling convention without enforcing it in the headers */
+#	if defined(WITH_FREETYPE)
+#		if !defined(FT_EXPORT)
+#			define FT_EXPORT( x )  extern "C"  x CDECL
+#		endif
+#	endif
 
+	#define strcasecmp stricmp
+	#define strncasecmp strnicmp
 	#define strtoull _strtoui64
 
 	/* MSVC doesn't have these :( */
@@ -285,35 +278,28 @@
 	#define SIGBUS SIGNOFP
 #endif
 
-#if defined(WINCE)
-	#define stredup _stredup
-#endif /* WINCE */
-
 /* NOTE: the string returned by these functions is only valid until the next
  * call to the same function and is not thread- or reentrancy-safe */
 #if !defined(STRGEN) && !defined(SETTINGSGEN)
-	#if defined(WIN32) || defined(WIN64)
+#	if defined(_WIN32)
 		char *getcwd(char *buf, size_t size);
 		#include <tchar.h>
 		#include <io.h>
 
-		/* XXX - WinCE without MSVCRT doesn't support wfopen, so it seems */
-		#if !defined(WINCE)
-			namespace std { using ::_tfopen; }
-			#define fopen(file, mode) _tfopen(OTTD2FS(file), _T(mode))
-			#define unlink(file) _tunlink(OTTD2FS(file))
-		#endif /* WINCE */
+		namespace std { using ::_tfopen; }
+		#define fopen(file, mode) _tfopen(OTTD2FS(file), _T(mode))
+		#define unlink(file) _tunlink(OTTD2FS(file))
 
 		const char *FS2OTTD(const TCHAR *name);
 		const TCHAR *OTTD2FS(const char *name, bool console_cp = false);
-	#else
+#	else
 		#define fopen(file, mode) fopen(OTTD2FS(file), mode)
 		const char *FS2OTTD(const char *name);
 		const char *OTTD2FS(const char *name);
-	#endif /* WIN32 */
+#	endif /* _WIN32 */
 #endif /* STRGEN || SETTINGSGEN */
 
-#if defined(WIN32) || defined(WIN64) || defined(__OS2__) && !defined(__INNOTEK_LIBC__)
+#if defined(_WIN32) || defined(__OS2__) && !defined(__INNOTEK_LIBC__)
 	#define PATHSEP "\\"
 	#define PATHSEPCHAR '\\'
 #else
@@ -321,6 +307,16 @@
 	#define PATHSEPCHAR '/'
 #endif
 
+#if defined(_MSC_VER) || defined(__WATCOMC__)
+#	define PACK_N(type_dec, n) __pragma(pack(push, n)) type_dec; __pragma(pack(pop))
+#elif defined(__MINGW32__)
+#	define PRAGMA(x) _Pragma(#x)
+#	define PACK_N(type_dec, n) PRAGMA(pack(push, n)) type_dec; PRAGMA(pack(pop))
+#else
+#	define PACK_N(type_dec, n) type_dec __attribute__((__packed__, aligned(n)))
+#endif
+#define PACK(type_dec) PACK_N(type_dec, 1)
+
 /* MSVCRT of course has to have a different syntax for long long *sigh* */
 #if defined(_MSC_VER) || defined(__MINGW32__)
 	#define OTTD_PRINTF64 "%I64d"
@@ -364,8 +360,7 @@ typedef unsigned char byte;
 /* Compile time assertions. Prefer c++0x static_assert().
  * Older compilers cannot evaluate some expressions at compile time,
  * typically when templates are involved, try assert_tcompile() in those cases. */
-#if defined(__STDCXX_VERSION__) || defined(__GXX_EXPERIMENTAL_CXX0X__) || defined(__GXX_EXPERIMENTAL_CPP0X__) || defined(static_assert)
-	/* __STDCXX_VERSION__ is c++0x feature macro, __GXX_EXPERIMENTAL_CXX0X__ is used by gcc, __GXX_EXPERIMENTAL_CPP0X__ by icc */
+#if __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600)
 	#define assert_compile(expr) static_assert(expr, #expr )
 	#define assert_tcompile(expr) assert_compile(expr)
 #elif defined(__OS2__)
